
import alice.tucson.api.*;
import alice.logictuple.*;

/**
 * Questa classe implementa il componente che comunica ad intervalli di tempo 
 * regolari a TuCSoN di commutare la luce del semaforo.
 * E' un Thread e non un Agent perch era necessario il metodo sleep(long t) non
 * disponibile per gli agenti TuCSoN.
 */
public class AgenteSemaforo extends Thread {
	
/**
 * Durata in secondi della luce verde.
 */
	protected final int TEMPO_SEMAFORO = 30;
	
	protected TucsonContext ctx;
	protected TupleCentreId semaforo;


/**
 * sim_time millisecondi del simulatore corrispondono ad 1 secondo reale.
 * All'aumentare di SIM_TIME diminuisce la velocit del simulatore.
 */	
	protected int sim_time;


/**
 * Inizializza il contesto e il centro di tuple.
 */
	public AgenteSemaforo(int t) throws Exception {
		ctx = Tucson.enterDefaultContext();
		semaforo = new TupleCentreId ("semaforo");
		sim_time = t;
	}


/**
 * Finch lo stato dell'ambiente semaforo  diverso da 'stop' gli ordina di 
 * cambiare la luce ogni TEMPO_SEMAFORO * sim_time millisecondi.
 */
	public void run() {
		Var state;
		try {
			do {
				sleep(TEMPO_SEMAFORO * sim_time);
				ctx.out (semaforo, new LogicTuple("cambia_luce"));
				state = new Var ("X");
				ctx.rd(semaforo, new LogicTuple("stato", state));
			} while (!state.toString().equals("stop"));

		} catch (Exception ex) {
			ex.printStackTrace ();
		}
	}
}
