
import alice.tucson.api.*;
import alice.logictuple.*;
import java.util.*;

/**
 * Classe che implementa il generatore di traffico e il componente che da il via
 * alla simulazione.
 * E' un Thread e non un Agent perch era necessario il metodo sleep(long t) non
 * disponibile per gli agenti TuCSoN.
 */
public class AgenteSimulatore extends Thread {
	
	protected TucsonContext ctx;
	protected TupleCentreId semaforo;
	protected TupleCentreId rotonda;

/**
 * sim_time millisecondi del simulatore corrispondono ad 1 secondo reale.
 * All'aumentare di SIM_TIME diminuisce la velocit del simulatore.
 */	
	protected int sim_time;

/**
 * Traffico in Erlang che si dovr generare.
 */	
	protected double traffic;
	
	protected Random rand;


/**
 * Inizializza il contesto e il centro di tuple e le altre variabili.
 */
	public AgenteSimulatore (int tm, double tf) throws Exception {
		ctx = Tucson.enterDefaultContext();
		semaforo = new TupleCentreId ("semaforo");
		rotonda = new TupleCentreId ("rotonda");		
		
		sim_time = tm;
		traffic = tf;

		rand = new Random(new Date().getTime());
	}


/**
 * Queto thread commuta lo stato della simulazione da 'stop' a 'start' e inizia 
 * a lanciare agenti automobile finch l'antisimulatore non commuta lo stato 
 * della simulazione.
 */
	public void run() {
		try {
			long k = 0;
			double tia = 0;
			boolean goodS, goodR;
			String origin, destination;
			AgenteAutomobile aas, aar;
			Var state;
			

						// INIZIO SIMULAZIONE
			ctx.out (semaforo, new LogicTuple("set_stato", new Value("start")));
			ctx.out (rotonda, new LogicTuple("set_stato", new Value("start")));


						// LANCIO SEMAFORO
			AgenteSemaforo as = new AgenteSemaforo(sim_time);
			as.setPriority(Thread.MAX_PRIORITY);
			as.start ();

			
						// GENERAZIONE DEL TRAFFICO
			do {
				// impostazione dei parametri dell'agente automobile
				origin = getRandomPosition();
				destination = getRandomPosition(origin);
				goodS = (Math.random() < 0.9);
				goodR = !((Math.random() < 0.3) && (adiacenti(origin, destination)));

				// lancio dell'agente automobile
				aas = new AgenteAutomobile("aas"+k, semaforo, origin, destination, goodS, sim_time);
				aar = new AgenteAutomobile("aar"+k, rotonda, origin, destination, goodR, sim_time);

				aas.spawn ();
				aar.spawn ();

				// attesa prima di un nuovo ciclo dipendente dal traffico
				tia = (-1/traffic) * Math.log(Math.random());
				ctx.out (semaforo, new LogicTuple("tempo_arrivo", new Value((long)(tia * sim_time))));
				sleep((long)(tia * sim_time));

				// lettura dello stato della simulazione
				state = new Var ("X");
				ctx.rd(semaforo, new LogicTuple("stato", state));
				k++;
			} while (state.toString().equals("start"));

		} catch (Exception ex) {
			ex.printStackTrace ();
		}
	}
	
	
	
/**
 * Genera una stringa casualmente che contiene una posizione tra nord, sud, ovest, est.
 */
	protected String getRandomPosition () {
		String pos = new String();
		
		int p = rand.nextInt (4);
		switch (p) {
			case 0: pos = "nord"; break;
			case 1: pos = "est"; break;
			case 2: pos = "sud"; break;
			case 3: pos = "ovest"; break;
		}
		
		return pos;
	}



/**
 * Genera una stringa casualmente che contiene una posizione tra nord, sud, ovest,
 * est ma diversa da quella passata a questo metodo come argomento.
 */
	protected String getRandomPosition (String o) {
		String pos;
		
		do {
			pos = getRandomPosition();
		} while (pos.equals (o));
		
		return pos;
	}
	
	
/**
 * Riceve due stringhe contenteti due posizioni.
 * Restituisce vero se e solo se esse sono adiacenti.
 */
	protected boolean adiacenti (String a, String b) {
		if ((a.equals("nord")) && (b.equals("ovest"))) return true;
		if ((a.equals("ovest")) && (b.equals("sud"))) return true;
		if ((a.equals("sud")) && (b.equals("est"))) return true;
		if ((a.equals("est")) && (b.equals("nord"))) return true;
		return false;
	}
}
