import java.awt.event.*;


/**
 * Classe che contiene le reazioni agli eventi dovuti alla pressione dei bottoni
 * dell'interfaccia grafica e che ha il compito di istanziare tutti l'interfaccia
 * grafica e l'interfaccia verso TuCSoN.
 */
public class Controllo extends WindowAdapter implements ActionListener {
	
	protected Interfaccia gui;
	protected Enviroment env;
	protected Report rep;


/**
 * Lancia l'interfaccia grafica e l'interfaccia verso TuCSoN
 */
	public Controllo () {
		gui = new Interfaccia(this);
		rep = new Report();

		try { 
			env = new Enviroment(gui);
			gui.DefaultConfiguration ();
		} catch (Exception ex) { 
			gui.printMessage(ex.getMessage ());
		}
	}



	public void actionPerformed (ActionEvent e) {
/**
 * Reazione al tasto START
 */
        if (e.getActionCommand().equals("START")) {
        	int n = Integer.MAX_VALUE;
   	    	try { 
   	    		if (gui.isAutomatic()) n = gui.getAgents();
	   	    	env.startSimulation(gui.getTraffic(), n);
    	   		gui.startConfiguration();
   	    	}
			catch (NumberFormatException ex) { 
				gui.printMessage("Numero di agenti non valido!");
			}
		}


/**
 * Reazione al tasto STOP
 */
        if (e.getActionCommand().equals("STOP")) {
   	    	env.stopSimulation();
       		gui.stopConfiguration();
        	rep.print(env.getStatistics());
        }


/**
 * Reazione al tasto RESET
 */
        if (e.getActionCommand().equals("RESET")) {
   	    	env.resetSimulation();
       		gui.stopConfiguration();
        	rep.print(env.getStatistics());
        }


/**
 * Reazione al tasto MOSTRA REPORT
 */
        if (e.getActionCommand().equals("MOSTRA REPORT")) {
        	rep.toggleVisibility();
        	rep.print(env.getStatistics());
        }
	

/**
 * Reazione al tasto AGGIORNA REPORT
 * Chiede statistiche aggiornate e le stampa
 */
		if (e.getActionCommand().equals("AGGIORNA REPORT")) {
			rep.print(env.getStatistics());
		}
	}
	
	
/**
 * Reazione alla chiusura della finestra
 */
	public void windowClosing (WindowEvent e) {
		gui.printMessage("Chiusura ambiente in corso...");
		System.out.println("Chiusura ambiente in corso...");
		try { env.exit (); }
		catch (Exception ex) { 
			System.out.println("Si e' verificato un errore durante l'uscita da TuCSoN:");
			System.out.println("Messaggio: " + ex.getMessage ());
		}
		System.exit(0);
	}
}
