
import alice.tucson.api.*;
import alice.logictuple.*;
import java.util.*;

import java.io.*;



/**
 * 
 */
public class Enviroment {

/**
 * SIM_TIME millisecondi del simulatore corrispondono ad 1 secondo reale.
 * All'aumentare di SIM_TIME diminuisce la velocit del simulatore.
 */	
	protected final int SIM_TIME = 100;
	
	protected TucsonContext ctx;
	protected TupleCentreId semaforo;
	protected TupleCentreId rotonda;
	protected Interfaccia gui;
	

	
/**
 * Controlla che TuCSoN sia in esecuzione e lancia l'agente che ha il compito di
 * creare gli ambienti semaforo e rotonda in TuCSoN.
 */
	public Enviroment(Interfaccia i) throws Exception {
		gui = i;
	
		// Imposta il contesto e i centri di tuple
		try {
			ctx = Tucson.enterDefaultContext();
			semaforo = new TupleCentreId ("semaforo");
			rotonda = new TupleCentreId ("rotonda");		
		} catch (Exception ex) { 
			throw new Exception("Errore durante l'accesso a TuCSoN!");
		}

		// Verifica il funzionamento di TuCSoN
		try {
			ctx.out (new TupleCentreId ("default"), new LogicTuple("prova"));
			ctx.inp (new TupleCentreId ("default"), new LogicTuple("prova"));
		} catch (Exception ex) {
			throw new Exception ("Avviare TuCSoN prima di eseguire questo programa!");
		}

		// Esegue un agente prolog per settare l'ambiente
		try {
			FileInputStream fis = new FileInputStream ("prolog/setup.pl");
			alice.tuprolog.Agent setupAgent = new alice.tuprolog.Agent(fis);
			setupAgent.spawn ();

			ctx.in (semaforo, new LogicTuple("setup", new Value("ok")));
		} catch (Exception ex) { 
			throw new Exception(ex.getMessage ());
		}
	}
			
	
	
/**
 * da il via alla simulazione lanciando l'agente simulatore e, solo se necessario,
 * l'agente per lo stop automatico
 */
	public void startSimulation(double traf, int nAgent) {
		try {
			AgenteSimulatore as = new AgenteSimulatore (SIM_TIME, traf);
			as.start ();
			
			if (nAgent < Integer.MAX_VALUE) {
				AgenteStopAutomatico ast = new AgenteStopAutomatico ("ast", nAgent, gui);
				ast.spawn ();
			}

		} catch (Exception ex) {
			ex.printStackTrace ();
		}
	}
	


/**
 * lancia l'agente anti simulatore per interrompere la simulazione
 */
	public void stopSimulation() {
		try {
			AgenteAntiSimulatore aas = new AgenteAntiSimulatore ("stop");
			aas.spawn ();
		} catch (Exception ex) {
			ex.printStackTrace ();
		}
	}



/**
 * lancia un agente prolog che inizializza i centri di tuple
 */
	public void resetSimulation() {
		try {
			FileInputStream fis = new FileInputStream ("prolog/reset.pl");
			alice.tuprolog.Agent exitAgent = new alice.tuprolog.Agent(fis);
			exitAgent.spawn ();

			ctx.in (semaforo, new LogicTuple("reset", new Value("ok")));
		} catch (Exception ex) {
			ex.printStackTrace ();
		}
	}
	
	
	
/**
 * Lancia un agente prolog che pulisce i centri di tuple
 */
	public void exit() throws Exception {
		FileInputStream fis = new FileInputStream ("prolog/exit.pl");
		alice.tuprolog.Agent exitAgent = new alice.tuprolog.Agent(fis);
		exitAgent.spawn ();

		ctx.in (semaforo, new LogicTuple("pulizia", new Value("ok")));
	}
	
	
	
/**
 * legge dai centri di tuple tutte le statistiche e restituisce un oggetto di
 * tipo Statistic.
 */
	public Statistics getStatistics () {
		Statistics s = new Statistics(SIM_TIME);
		
		try {
			Var ui = new Var("UI");
			Var ti = new Var("TI");
			Var uss = new Var("USS");
			Var tss = new Var("TSS");
			Var usr = new Var("USR");
			Var tsr = new Var("TSR");
			Var cs = new Var("CS");
			Var cr = new Var("CR");
		
			ctx.rd (semaforo, new LogicTuple("statistiche", new Value("utenti_arrivati"), ui));
			ctx.rd (semaforo, new LogicTuple("statistiche", new Value("tempo_arrivo"), ti));
			ctx.rd (semaforo, new LogicTuple("statistiche", new Value("utenti_serviti"), uss));
			ctx.rd (semaforo, new LogicTuple("statistiche", new Value("tempo_servizio"), tss));
			ctx.rd (semaforo, new LogicTuple("statistiche", new Value("coda_max"), cs));
			ctx.rd (rotonda, new LogicTuple("statistiche", new Value("utenti_serviti"), usr));
			ctx.rd (rotonda, new LogicTuple("statistiche", new Value("tempo_servizio"), tsr));
			ctx.rd (rotonda, new LogicTuple("statistiche", new Value("coda_max"), cr));
			
			s.setArrivedUser (Long.parseLong (ui.toString()));
			s.setTotalArrivalTime (Long.parseLong (ti.toString()));
			s.setServedUserSmf (Long.parseLong (uss.toString()));
			s.setTotalServiceTimeSmf (Long.parseLong (tss.toString()));
			s.setMaxQueueSmf(Long.parseLong (cs.toString()));
			s.setServedUserRtd (Long.parseLong (usr.toString()));
			s.setTotalServiceTimeRtd (Long.parseLong (tsr.toString()));
			s.setMaxQueueRtd(Long.parseLong (cr.toString()));
			
		} catch (Exception ex) {
			ex.printStackTrace ();
		}
		return s;
	}
}
