import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


/**
 * Interfaccia grafica principale del programma
 */
public class Interfaccia implements ChangeListener, ActionListener {
	
    protected Controllo listener;
    protected JButton start;
    protected JButton stop;
    protected JButton reset;
    protected JButton show;
    protected JButton update;
    protected JLabel outputDevice;
    protected JCheckBox auto;
    protected JSlider slider;
    protected JLabel traffic;
    protected JTextField agents;
    protected JLabel text_agenti;
    protected JLabel text_traffico;
    

/**
 * Inizializza il listener degli eventi e costruisce l'interfaccia
 */
    public Interfaccia(Controllo c) {
		listener = c;
		setUp();
    }


/**
 * Costruisce e mostra l'interfaccia
 */
	protected void setUp() {
        JFrame console = new JFrame("Simulatore di semafori e rotonde");
        console.addWindowListener (listener);
        Container container = console.getContentPane();
		container.setLayout(new BorderLayout());
		
		start = new JButton("START");
		start.addActionListener (listener);
		start.setEnabled (false);
		
		stop = new JButton("STOP");
		stop.addActionListener (listener);
		stop.setEnabled (false);

		reset = new JButton("RESET");
		reset.addActionListener (listener);
		reset.setEnabled (false);

		show = new JButton("MOSTRA REPORT");
		show.addActionListener (listener);
		show.setEnabled (false);

		update = new JButton("AGGIORNA REPORT");
		update.addActionListener (listener);
		update.setEnabled (false);

		JPanel setup = new JPanel();
		JPanel buttonRow1 = new JPanel();
		JPanel buttonRow2 = new JPanel();

		setup.setLayout(new GridLayout(2,1));
		buttonRow1.setLayout (new GridLayout(1,3));
		buttonRow2.setLayout (new GridLayout(1,2));

		JPanel sliderPanel = new JPanel();
		JPanel checkPanel = new JPanel();
		
		slider = new JSlider();
        slider.setMinimum(1);
        slider.setMaximum(10);
        slider.setValue(2);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setSnapToTicks(true);
        slider.addChangeListener(this);
		traffic = new JLabel(" 0.2 Erlang");
		text_traffico = new JLabel("Traffico: ");
		
		sliderPanel.add(text_traffico);
		sliderPanel.add(slider);
		sliderPanel.add(traffic);
		
		auto = new JCheckBox ("Interrompi automaticamente dopo");
		auto.addActionListener(this);
		agents = new JTextField(4);
		agents.setText("100");
		agents.setEditable(false);
		text_agenti = new JLabel(" agenti");
		
		checkPanel.add(auto);
		checkPanel.add(agents);
		checkPanel.add(text_agenti);
		
		setup.add(sliderPanel);
		setup.add(checkPanel);
		
		buttonRow1.add (start);
		buttonRow1.add (stop);
		buttonRow1.add (reset);
		buttonRow2.add (show);
		buttonRow2.add (update);
		
		JPanel centerCont = new JPanel();
		centerCont.setLayout (new GridLayout(3,1));
		centerCont.add (setup);
		centerCont.add (buttonRow1);
		centerCont.add (buttonRow2);

		outputDevice = new JLabel("Creazione ambiente in corso...");
		
		container.add (centerCont, BorderLayout.CENTER);
		container.add (outputDevice, BorderLayout.SOUTH);

        console.setSize(350, 250);
		console.setVisible (true);
	}
	
	
	
/**
 * Configurazione iniziale dei componenti
 */
	public void DefaultConfiguration() {
		start.setEnabled (true);
		stop.setEnabled (false);
		reset.setEnabled (false);
		show.setEnabled (true);
		update.setEnabled (false);
		text_traffico.setEnabled (true);
		slider.setEnabled (true);
		traffic.setEnabled (true);
		auto.setEnabled (true);
		agents.setEnabled (true);
		text_agenti.setEnabled (true);
		printMessage ("Premi START per iniziare la simulazione...");
	}


/**
 * Configurazione dei componenti dopo la pressione del tasto START
 */
	public void startConfiguration() {
		start.setEnabled (false);
		stop.setEnabled (!auto.isSelected());
		reset.setEnabled (false);
		show.setEnabled (true);
		update.setEnabled (true);
		text_traffico.setEnabled (false);
		slider.setEnabled (false);
		traffic.setEnabled (false);
		auto.setEnabled (false);
		agents.setEnabled (false);
		text_agenti.setEnabled (false);
       	printMessage("Simulazione in corso...");
	}
	
	
/**
 * Configurazione dei componenti dopo la pressione del tasto STOP
 */
	public void stopConfiguration() {
		start.setEnabled (true);
		stop.setEnabled (false);
		reset.setEnabled (true);
		show.setEnabled (true);
		update.setEnabled (true);
		text_traffico.setEnabled (true);
		slider.setEnabled (true);
		traffic.setEnabled (true);
		auto.setEnabled (true);
		agents.setEnabled (true);
		text_agenti.setEnabled (true);
	   	printMessage("Simulazione terminata!");
	}


/**
 * Stampa un messaggio nella riga in basso
 */
	public void printMessage(String m) {
		outputDevice.setText (m);
	}
	
	
/**
 * Restituisce il valore di traffico impostato con la barra (slider)
 */
	public double getTraffic() {
		return ((double)(slider.getValue())/10);
	}
	
	
/**
 * Indica se il check box  selezionato o meno
 */
	public boolean isAutomatic () {
		return auto.isSelected();
	}
	
	
/**
 * Restituisce il numero di agenti impostato per lo stop autometico
 */
	public int getAgents() throws NumberFormatException {
		return new Integer(agents.getText()).intValue();
	}


/**
 * Reazione all'evento di cambiamento della barra (slider)
 */
    public void stateChanged(ChangeEvent e) {
		traffic.setText(" " + ((float)(slider.getValue())/10) + " Erlang");
    }


/**
 * Reazione alla selezione del check box
 */
    public void actionPerformed(ActionEvent e){
        if (e.getSource().equals(auto)) {
        	agents.setEditable(auto.isSelected());
		}
	}
}
